searchNodes=[{"ref":"Mix.Tasks.Cloak.Migrate.Ecto.html","title":"Mix.Tasks.Cloak.Migrate.Ecto","type":"task","doc":"Migrates a schema table to a new encryption cipher. Rationale Cloak vaults will automatically decrypt fields which were encrypted by a retired key, and reencrypt them with the new key when they change. However, this usually is not enough for key rotation. Usually, you want to proactively reencrypt all your fields with the new key, so that the old key can be decommissioned. This task allows you to do just that. Strategy This task will migrate a table following this strategy: Page through the table using a cursor, selecting primary key values. By default, the cursor is based on the schema&#39;s primary key. To use custom fields in the cursor, see Cloak.CustomCursor. Here&#39;s an example SQL query generated by mix cloak.migrate.ecto: SELECT primary_key FROM table_name WHERE primary_key &gt; cursor ORDER BY primary_key ASC LIMIT 100 For each primary key value, in a database transaction, Fetch the row with that key value, lock with &quot;FOR UPDATE&quot; Reencrypt all Cloak fields with the new cipher Write the row, unlocking it Update queries are issued in parallel to maximize speed. Each row is fetched and written back as quickly as possible to reduce the amount of time the row is locked. Warnings mix cloak.migrate.ecto works well with primary keys of the following types. If your key is not of these types, see Cloak.CustomCursor. Integers PostgreSQL UUID MongoDB ObjectID Because mix cloak.migrate.ecto issues queries in parallel, it can consume all your database connections. For this reason, you may wish to use a separate Repo with a limited :pool just for Cloak migrations. This will allow you to prevent any performance impact by throttling Cloak to use only a limited number of database connections. Configuration Ensure that you have configured your vault to use the new cipher by default! # If using mix configuration... config :my_app, MyApp.Vault, ciphers: [ default: {Cloak.Ciphers.AES.GCM, tag: &quot;NEW&quot;, key: &lt;&lt;...&gt;&gt;}, retired: {Cloak.Ciphers.AES.CTR, tag: &quot;OLD&quot;, key: &lt;&lt;...&gt;&gt;&gt;} ] # If configuring in the `init/1` callback: defmodule MyApp.Vault do use Cloak.Vault, otp_app: :my_app @impl Cloak.Vault def init(config) do config = Keyword.put(config, :ciphers, [ default: {Cloak.Ciphers.AES.GCM, tag: &quot;NEW&quot;, key: &lt;&lt;...&gt;&gt;}, retired: {Cloak.Ciphers.AES.CTR, tag: &quot;OLD&quot;, key: &lt;&lt;...&gt;&gt;&gt;} ]) {:ok, config} end end If you want to migrate multiple schemas at once, you may find it convenient to specify the schemas in your config/config.exs: config :my_app, cloak_repo: [MyApp.Repo], cloak_schemas: [MyApp.Schema1, MyApp.Schema2] Usage To run against only a specific repo and schema, use the -r and -s flags: mix cloak.migrate.ecto -r MyApp.Repo -s MyApp.Schema If you&#39;ve configured multiple schemas at once, as shown above, you can simply run: mix cloak.migrate.ecto"},{"ref":"Cloak.Ecto.CustomCursor.html","title":"Cloak.Ecto.CustomCursor","type":"behaviour","doc":"Defines a list of fields to use as a cursor when migrating a schema&#39;s table to a new encryption key using mix cloak.migrate.ecto. When to Use This This behaviour is useful in rare situations where your primary key cannot be used as a cursor to page through the table. You need this if primary key cannot meet the &quot;Column Requirements&quot; below. For example, you don&#39;t need to use this module if your primary key is: An integer A PostgreSQL UUID A MongoDB ObjectID Your primary key already meets the &quot;Column Requirements&quot; Column Requirements Each column you specify for the cursor must meet the following requirements: Each column must contain sortable values Each column must be comparable using the &gt; operator The combined value of the columns must be unique in the table Example defmodule MyApp.MySchema do use Ecto.Schema @behaviour Cloak.Ecto.CustomCursor schema &quot;table&quot; do # ... end @impl Cloak.CustomCursor def __cloak_cursor_fields__ do [:sequential_id, :inserted_at] end end"},{"ref":"Cloak.Ecto.CustomCursor.html#c:__cloak_cursor_fields__/0","title":"Cloak.Ecto.CustomCursor.__cloak_cursor_fields__/0","type":"callback","doc":"Returns a list of cursor fields. Example @impl Cloak.Ecto.CustomCursor def __cloak_cursor_fields__ do [:sequential_id, :inserted_at] end"},{"ref":"Cloak.Ecto.Binary.html","title":"Cloak.Ecto.Binary","type":"module","doc":"An Ecto.Type to encrypt a binary field. Migration The database field must be of type :binary. add :encrypted_field, :binary Usage Define an Encrypted.Binary module in your project: defmodule MyApp.Encrypted.Binary do use Cloak.Ecto.Binary, vault: MyApp.Vault end Then, define the type of your desired fields: schema &quot;table_name&quot; do field :encrypted_field, MyApp.Encrypted.Binary end"},{"ref":"Cloak.Ecto.Date.html","title":"Cloak.Ecto.Date","type":"module","doc":"An Ecto.Type to encrypt Date fields. Migration The database field must be of type :binary. add :encrypted_field, :binary Usage Define an Encrypted.Date module in your project: defmodule MyApp.Encrypted.Date do use Cloak.Ecto.Date, vault: MyApp.Vault end Then, define the type of your desired fields: schema &quot;table_name&quot; do field :encrypted_field, MyApp.Encrypted.Date end"},{"ref":"Cloak.Ecto.DateTime.html","title":"Cloak.Ecto.DateTime","type":"module","doc":"An Ecto.Type to encrypt DateTime fields. Migration The database field must be of type :binary. add :encrypted_field, :binary Usage Define an Encrypted.DateTime module in your project: defmodule MyApp.Encrypted.DateTime do use Cloak.Ecto.DateTime, vault: MyApp.Vault end Then, define the type of your desired fields: schema &quot;table_name&quot; do field :encrypted_field, MyApp.Encrypted.DateTime end"},{"ref":"Cloak.Ecto.Float.html","title":"Cloak.Ecto.Float","type":"module","doc":"An Ecto.Type to encrypt a float field. Migration The database field must be of type :binary. add :encrypted_field, :binary Usage Define an Encrypted.Float module in your project: defmodule MyApp.Encrypted.Float do use Cloak.Ecto.Float, vault: MyApp.Vault end Then, define the type of your desired fields: schema &quot;table_name&quot; do field :encrypted_field, MyApp.Encrypted.Float end"},{"ref":"Cloak.Ecto.HMAC.html","title":"Cloak.Ecto.HMAC","type":"behaviour","doc":"A custom Ecto.Type for hashing fields using :crypto.hmac/3. Why If you store a hash of a field&#39;s value, you can then query on it as a proxy for an encrypted field. This works because HMAC is deterministic and always results in the same value, while secure encryption does not. Be warned, however, that hashing will expose which fields have the same value, because they will contain the same hash. Security HMAC is more secure than Cloak.Ecto.SHA256, because it uses a secret to obfuscate the hash. This makes it harder to guess the value of the field. Configuration Create an HMAC field in your project: defmodule MyApp.Hashed.HMAC do use Cloak.Ecto.HMAC, otp_app: :my_app end Then, configure it with a :secret and :algorithm, either using mix configuration: config :my_app, MyApp.Hashed.HMAC, algorithm: :sha512, secret: &quot;secret&quot; Or using the init/1 callback to fetch configuration at runtime: defmodule MyApp.Hashed.HMAC do use Cloak.Ecto.HMAC, otp_app: :my_app @impl Cloak.Ecto.HMAC def init(config) do config = Keyword.merge(config, [ algorithm: :sha512, secret: System.get_env(&quot;HMAC_SECRET&quot;) ]) {:ok, config} end end Usage Create the hash field with the type :binary. Add it to your schema definition like this: schema &quot;table&quot; do field :field_name, MyApp.Encrypted.Binary field :field_name_hash, MyApp.Hashed.HMAC end Ensure that the hash is updated whenever the target field changes with the put_change/3 function: def changeset(struct, attrs \\\\ %{}) do struct |&gt; cast(attrs, [:field_name, :field_name_hash]) |&gt; put_hashed_fields() end defp put_hashed_fields(changeset) do changeset |&gt; put_change(:field_name_hash, get_field(changeset, :field_name)) end Query the Repo using the :field_name_hash in any place you would typically query by :field_name. user = Repo.get_by(User, email_hash: &quot;user@email.com&quot;)"},{"ref":"Cloak.Ecto.HMAC.html#c:init/1","title":"Cloak.Ecto.HMAC.init/1","type":"callback","doc":"Configures the HMAC field using runtime information. Example @impl Cloak.Fields.HMAC def init(config) do config = Keyword.merge(config, [ algorithm: :sha512, secret: System.get_env(&quot;HMAC_SECRET&quot;) ]) {:ok, config} end"},{"ref":"Cloak.Ecto.HMAC.html#t:algorithms/0","title":"Cloak.Ecto.HMAC.algorithms/0","type":"type","doc":"HMAC algorithms supported by Cloak.Field.HMAC"},{"ref":"Cloak.Ecto.Integer.html","title":"Cloak.Ecto.Integer","type":"module","doc":"An Ecto.Type to encrypt integer fields. Migration The database field must be of type :binary. add :encrypted_field, :binary Usage Define an Encrypted.Integer module in your project: defmodule MyApp.Encrypted.Integer do use Cloak.Ecto.Integer, vault: MyApp.Vault end Then, define the type of your desired fields: schema &quot;table_name&quot; do field :encrypted_field, MyApp.Encrypted.Integer end"},{"ref":"Cloak.Ecto.IntegerList.html","title":"Cloak.Ecto.IntegerList","type":"module","doc":"An Ecto.Type to encrypt a list of integers. Configuration You can customize the json library used for for converting the lists. config :my_app, MyApp.Vault, json_library: Jason Migration The database field must be of type :binary. add :encrypted_field, :binary Usage Define an Encrypted.IntegerList module in your project: defmodule MyApp.Encrypted.IntegerList do use Cloak.Ecto.IntegerList, vault: MyApp.Vault end Then, define the type of your desired fields: schema &quot;table_name&quot; do field :encrypted_field, MyApp.Encrypted.IntegerList end"},{"ref":"Cloak.Ecto.Map.html","title":"Cloak.Ecto.Map","type":"module","doc":"An Ecto.Type to encrypt maps. Configuration You can customize the json library used for for converting the maps to and from JSON during encryption and decryption. config :my_app, MyApp.Vault, json_library: Jason Migration The database field must be of type :binary. add :encrypted_field, :binary Usage Define an Encrypted.Map module in your project: defmodule MyApp.Encrypted.Map do use Cloak.Ecto.Map, vault: MyApp.Vault end Then, define the type of your desired fields: schema &quot;table_name&quot; do field :encrypted_field, MyApp.Encrypted.Map end On encryption, the map will first be converted to JSON using the configured :json_library, and then encrypted. On decryption, the :json_library will be used to convert it back to a map. This means that on decryption, atom keys will become string keys. # ON WRITE %{hello: &quot;world&quot;} # ON READ # Keys converted to strings %{&quot;hello&quot; =&gt; &quot;world&quot;}"},{"ref":"Cloak.Ecto.NaiveDateTime.html","title":"Cloak.Ecto.NaiveDateTime","type":"module","doc":"An Ecto.Type to encrypt NaiveDateTime fields. Migration The database field must be of type :binary. add :encrypted_field, :binary Usage Define an Encrypted.NaiveDateTime module in your project: defmodule MyApp.Encrypted.NaiveDateTime do use Cloak.Ecto.NaiveDateTime, vault: MyApp.Vault end Then, define the type of your desired fields: schema &quot;table_name&quot; do field :encrypted_field, MyApp.Encrypted.NaiveDateTime end"},{"ref":"Cloak.Ecto.PBKDF2.html","title":"Cloak.Ecto.PBKDF2","type":"behaviour","doc":"A custom Ecto.Type for deriving a key for fields using PBKDF2. PBKDF2 is more secure than Cloak.Ecto.HMAC and Cloak.Fields.SHA256 because it uses key stretching to increase the amount of time to compute hashes. This slows down brute-force attacks. Why If you store a hash of a field&#39;s value, you can then query on it as a proxy for an encrypted field. This works because PBKDF2 is deterministic and always results in the same value, while secure encryption does not. Be warned, however, that hashing will expose which fields have the same value, because they will contain the same hash. Dependency To use this field type, you must install the :pbkdf2 library in your mix.exs file. {:pbkdf2, &quot;~&gt; 2.0&quot;} Configuration Create a PBKDF2 field in your project: defmodule MyApp.Hashed.PBKDF2 do use Cloak.Ecto.PBKDF2, otp_app: :my_app end Then, configure it with a :secret, an :algorithm, the maximum :size of the stored key (in bytes), and a number of :iterations, either using mix configuration: config :my_app, MyApp.Hashed.PBKDF2, algorithm: :sha256, iterations: 10_000, secret: &quot;secret&quot;, size: 64 Or using the init/1 callback to fetch configuration at runtime: defmodule MyApp.Hashed.PBKDF2 do use Cloak.Ecto.PBKDF2, otp_app: :my_app @impl Cloak.Ecto.PBKDF2 def init(config) do config = Keyword.merge(config, [ algorithm: :sha256, iterations: 10_000, secret: System.get_env(&quot;PBKDF2_SECRET&quot;) ]) {:ok, config} end end Usage Create the hash field with the type :binary. Add it to your schema definition like this: schema &quot;table&quot; do field :field_name, MyApp.Encrypted.Binary field :field_name_hash, MyApp.Hashed.PBKDF2 end Ensure that the hash is updated whenever the target field changes with the put_change/3 function: def changeset(struct, attrs \\\\ %{}) do struct |&gt; cast(attrs, [:field_name, :field_name_hash]) |&gt; put_hashed_fields() end defp put_hashed_fields(changeset) do changeset |&gt; put_change(:field_name_hash, get_field(changeset, :field_name)) end Query the Repo using the :field_name_hash in any place you would typically query by :field_name. user = Repo.get_by(User, email_hash: &quot;user@email.com&quot;)"},{"ref":"Cloak.Ecto.PBKDF2.html#c:init/1","title":"Cloak.Ecto.PBKDF2.init/1","type":"callback","doc":"Configures the PBKDF2 field using runtime information. Example @impl Cloak.Ecto.PBKDF2 def init(config) do config = Keyword.merge(config, [ algorithm: :sha256, secret: System.get_env(&quot;PBKDF2_SECRET&quot;) ]) {:ok, config} end"},{"ref":"Cloak.Ecto.PBKDF2.html#t:algorithms/0","title":"Cloak.Ecto.PBKDF2.algorithms/0","type":"type","doc":"Digest algorithms supported by Cloak.Field.PBKDF2"},{"ref":"Cloak.Ecto.SHA256.html","title":"Cloak.Ecto.SHA256","type":"module","doc":"An Ecto.Type which hashes the field value using the SHA256 algorithm. Why If you store a hash of a field&#39;s value, you can then query on it as a proxy for the encrypted field. This works because SHA256 is deterministic and always results in the same value, while secure encryption does not. Be warned, however, that hashing will expose which fields have the same value, because they will contain the same hash. Security For a more secure hashing method, see one of the following alternatives: Cloak.Ecto.HMAC Cloak.Ecto.PBKDF2 Usage Create the hash field with the type :binary. Add it to your schema definition like this: schema &quot;table&quot; do field :field_name, MyApp.Encrypted.Binary field :field_name_hash, Cloak.Ecto.SHA256 end Ensure that the hash is updated whenever the target field changes with the put_change/3 function: def changeset(struct, attrs \\\\ %{}) do struct |&gt; cast(attrs, [:field_name, :field_name_hash]) |&gt; put_change(:field_name_hash, get_field(changeset, :field_name)) end Query the Repo using the :field_name_hash in any place you would typically query by :field_name. user = Repo.get_by(User, email_hash: &quot;user@email.com&quot;)"},{"ref":"Cloak.Ecto.StringList.html","title":"Cloak.Ecto.StringList","type":"module","doc":"An Ecto.Type to encrypt a list of strings. Configuration You can customize the json library used for for converting lists. config :my_app, MyApp.Vault, json_library: Jason Migration The database field must be of type :binary. add :encrypted_field, :binary Usage Define an Encrypted.StringList module in your project: defmodule MyApp.Encrypted.StringList do use Cloak.Ecto.StringList, vault: MyApp.Vault end Then, define the type of your desired fields: schema &quot;table_name&quot; do field :encrypted_field, MyApp.Encrypted.StringList end On encryption, the list will first be converted to JSON using the configured :json_library, and then encrypted. On decryption, the :json_library will be used to convert it back to a list of strings."},{"ref":"Cloak.Ecto.Time.html","title":"Cloak.Ecto.Time","type":"module","doc":"An Ecto.Type to encrypt Time fields. Migration The database field must be of type :binary. add :encrypted_field, :binary Usage Define an Encrypted.Time module in your project: defmodule MyApp.Encrypted.Time do use Cloak.Ecto.Time, vault: MyApp.Vault end Then, define the type of your desired fields: schema &quot;table_name&quot; do field :encrypted_field, MyApp.Encrypted.Time end"},{"ref":"readme.html","title":"Cloak.Ecto","type":"extras","doc":"Cloak.Ecto Easily encrypt fields in your Ecto schemas. Relies on Cloak for encryption. Hex Documentation"},{"ref":"readme.html#usage","title":"Cloak.Ecto - Usage","type":"extras","doc":"Cloak.Ecto helps you create Ecto.Type modules which automatically encrypt and decrypt your data. You simply set the type of your fields, and Cloak.Ecto handles the rest. defmodule MyApp.EctoSchema do use Ecto.Schema schema &quot;table_name&quot; do field :encrypted_field, MyApp.Encrypted.Binary # ... end end When Ecto writes the fields to the database, Cloak encrypts the values into a binary blob, using a configured encryption algorithm chosen by you. iex&gt; Repo.insert!(%MyApp.EctoSchema{encrypted_field: &quot;plaintext&quot;}) 08:46:08.862 [debug] QUERY OK db=3.4ms INSERT INTO &quot;table_name&quot; (&quot;encrypted_field&quot;) VALUES ($1) RETURNING &quot;id&quot;, &quot;encrypted_field&quot; [ &lt;&lt;1,10, 65, 69, 83, 46, 67, 84, 82, 46, 86, 49, 69, 92, 173, 219, 203, 238, 26, 58, 236, 5, 104, 23, 12, 10, 182, 31, 221, 89, 22, 58, 34, 79, 109, 30, 70, 254, 56, 93, 102, 84&gt;&gt; ] Likewise, when Ecto reads the encrypted blob out of the database, Cloak will automatically decrypt the value into the intended data type at runtime. iex&gt; Repo.get(MyApp.EctoSchema, 1) %MyApp.EctoSchema{encrypted_field: &quot;plaintext&quot;} For complete usage instructions, see the Hex documentation."},{"ref":"readme.html#notable-features","title":"Cloak.Ecto - Notable Features","type":"extras","doc":"Transparent, easy to use encryption for database fields Fully compatible with umbrella projects Bring your own encryption algorithm, if you want Mix task for key rotation: mix cloak.migrate"},{"ref":"readme.html#security-notes","title":"Cloak.Ecto - Security Notes","type":"extras","doc":"Supported Algorithms: Cloak&#39;s built-in encryption modules rely on Erlang&#39;s :crypto module. Cloak supports the following algorithms out of the box: AES.GCM AES.CTR Encrypted Data Not Searchable: Cloak uses random IVs for each ciphertext. This means that the same value will not encrypt to the same value twice. As a result, encrypted columns are not queryable. However, Cloak does provide easy ways to create hashed, searchable columns. Runtime Data is not Encrypted: Cloak encrypts data at rest in the database. The data in your Ecto structs at runtime is not encrypted. No Support for User-specific Encryption Keys: Cloak&#39;s Ecto.Type modules do not support user-specific encryption keys, due to limitations on the Ecto.Type behaviour. However, you can still use Cloak&#39;s ciphers to implement these in your application logic."},{"ref":"readme.html#use-without-ecto","title":"Cloak.Ecto - Use Without Ecto","type":"extras","doc":"If you want to use Cloak without Ecto, see cloak instead."},{"ref":"install.html","title":"Install Cloak","type":"extras","doc":"Installing Cloak.Ecto This guide will walk you through installing Cloak.Ecto in your project. Add the Dependency First, add :cloak_ecto to your dependencies in mix.exs: {:cloak_ecto, &quot;~&gt; 1.0.0&quot;} Run mix deps.get to fetch the dependency. Since :cloak_ecto relies on :cloak, the :cloak package will also be installed. Generate a Key You&#39;ll need a secret key for encryption. This is easy to generate in the IEx console. $ iex iex&gt; 32 |&gt; :crypto.strong_rand_bytes() |&gt; Base.encode64() &quot;aJ7HcM24BcyiwsAvRsa3EG3jcvaFWooyQJ+91OO7bRU=&quot; This will generate a relatively strong encryption 256-bit encryption key encoded with Base64. Create a Vault Next, create a Cloak.Vault for your project. defmodule MyApp.Vault do use Cloak.Vault, otp_app: :my_app end Configure it as shown in the Cloak.Vault documentation, with at least one active cipher. Note that the :key needs to be decoded from Base64 encoding into its raw binary form. config :my_app, MyApp.Vault, ciphers: [ default: {Cloak.Ciphers.AES.GCM, tag: &quot;AES.GCM.V1&quot;, key: Base.decode64!(&quot;your-key-here&quot;)} ] If you want to fetch keys from system vars, you should use the init/1 callback to configure the vault instead: # Assumes that you have a CLOAK_KEY environment variable containing a key in # Base64 encoding. # # export CLOAK_KEY=&quot;A7x+qcFD9yeRfl3GohiOFZM5bNCdHNu27B0Ozv8X4dE=&quot; defmodule MyApp.Vault do use Cloak.Vault, otp_app: :my_app @impl GenServer def init(config) do config = Keyword.put(config, :ciphers, [ default: {Cloak.Ciphers.AES.GCM, tag: &quot;AES.GCM.V1&quot;, key: decode_env!(&quot;CLOAK_KEY&quot;)} ]) {:ok, config} end defp decode_env!(var) do var |&gt; System.get_env() |&gt; Base.decode64!() end end Finally, add your vault to your supervision tree. children = [ MyApp.Vault ] Create Local Ecto Types For each type of data you want to encrypt, define a local Ecto type: defmodule MyApp.Encrypted.Binary do use Cloak.Ecto.Binary, vault: MyApp.Vault end You can find a complete list of available types in the &quot;MODULES&quot; documentation. Create Your Schema If you want to encrypt an existing schema, see the guide on Encrypting Existing Data. If you&#39;re starting from scratch with a new Ecto.Schema, generate your fields with the type :binary: create table(:users) do add :email, :binary add :email_hash, :binary # will be used for searching # ... timestamps() end Your schema module should look like this: defmodule MyApp.Accounts.User do use Ecto.Schema import Ecto.Changeset schema &quot;users&quot; do field :email, MyApp.Encrypted.Binary field :email_hash, Cloak.Ecto.SHA256 # ... other fields timestamps() end @doc false def changeset(struct, attrs \\\\ %{}) do struct |&gt; cast(attrs, [:email]) |&gt; put_hashed_fields() end defp put_hashed_fields(changeset) do changeset |&gt; put_change(:email_hash, get_field(changeset, :email)) end end This example also shows how you would make a given field queryable by creating a mirrored _hash field. See Cloak.Ecto.SHA256 or Cloak.Ecto.HMAC for more details."},{"ref":"install.html#usage","title":"Install Cloak - Usage","type":"extras","doc":"Your encrypted fields will be transparently encrypted and decrypted as data are loaded from the database. Repo.get(Accounts.User, 1) # =&gt; %Accounts.User{email: &quot;test@example.com&quot;, email_hash: &lt;&lt;115, 6, 45, 135, 41, ...&gt;&gt;} You can query by the mirrored _hash fields: Repo.get_by(Accounts.User, email_hash: &quot;test@example.com&quot;) # =&gt; %Accounts.User{email: &quot;test@example.com&quot;, ...} And you&#39;re done! Cloak.Ecto is successfully installed."},{"ref":"generate_keys.html","title":"Generate Encryption Keys","type":"extras","doc":"How to Generate Encryption Keys The easiest way to generate a key is via IEx. $ iex iex&gt; 32 |&gt; :crypto.strong_rand_bytes() |&gt; Base.encode64() &quot;HXCdm5z61eNgUpnXObJRv94k3JnKSrnfwppyb60nz6w=&quot; This will generate a strong 256-bit key encoded with Base64."},{"ref":"encrypt_existing_data.html","title":"Encrypt Existing Data","type":"extras","doc":"How to Encrypt Existing Data When you adopt Cloak in an existing project, you often need to encrypt fields that were not previously encrypted. This guide will help you through the process, step by step."},{"ref":"encrypt_existing_data.html#create-a-vault","title":"Encrypt Existing Data - Create a Vault","type":"extras","doc":"Make sure you&#39;ve set up (and configured!) a vault as described in the Cloak.Vault documentation and the installation guide."},{"ref":"encrypt_existing_data.html#create-local-ecto-types","title":"Encrypt Existing Data - Create Local Ecto Types","type":"extras","doc":"Create local Ecto types for each kind of field you want to encrypt, as described in the Cloak.Vault documentation and the installation guide."},{"ref":"encrypt_existing_data.html#add-encrypted_-fields","title":"Encrypt Existing Data - Add encrypted_* Fields","type":"extras","doc":"You&#39;ll need to create a duplicate version of each field you want to encrypt, perhaps with the encrypted_ prefix. alter table(:users) do add :encrypted_name, :binary add :encrypted_metadata, :binary end Then, add the new fields to your schema. Make sure to keep them up to date with changes using a temporary put_encrypted_fields function in your changeset. defmodule MyApp.Accounts.User do use Ecto.Schema import Ecto.Changeset schema &quot;users&quot; do field :name, :string field :encrypted_name, MyApp.Encrypted.Binary field :metadata, :map field :encrypted_metadata, MyApp.Encrypted.Map end @doc false def changeset(struct, attrs \\\\ %{}) do struct |&gt; cast(attrs, [:name, :metadata]) |&gt; put_encrypted_fields() end # Temporary function during the migration process, to ensure # that all changes are copied over to the new fields defp put_encrypted_fields(changeset) do changeset |&gt; put_change(:encrypted_name, get_field(changeset, :name)) |&gt; put_change(:encrypted_metadata, get_field(changeset, :metadata)) end end"},{"ref":"encrypt_existing_data.html#copy-data-to-new-fields","title":"Encrypt Existing Data - Copy Data to New Fields","type":"extras","doc":"In a custom mix task or the IEx console, run the following: MyApp.Accounts.User |&gt; MyApp.Repo.all() |&gt; Enum.map(fn user -&gt; user |&gt; Ecto.Changeset.change(%{ encrypted_name: user.name, encrypted_metadata: user.metadata }) |&gt; MyApp.Repo.update!() end)"},{"ref":"encrypt_existing_data.html#remove-original-fields","title":"Encrypt Existing Data - Remove Original Fields","type":"extras","doc":"Once you&#39;re confident that all the data has migrated successfully to the new fields, you can write a migration to remove the old, unencrypted fields. alter table(:users) do remove :name remove :metadata end rename table(:users), :encrypted_name, to: :name rename table(:users), :encrypted_metadata, to: :metadata You can also remove the temporary put_encrypted_fields/1 function, leaving your schema like this: defmodule MyApp.Accounts.User do use Ecto.Schema import Ecto.Changeset schema &quot;users&quot; do field :name, MyApp.Encrypted.Binary field :metadata, MyApp.Encrypted.Map end @doc false def changeset(struct, attrs \\\\ %{}) do struct |&gt; cast(attrs, [:name, :metadata]) end end"},{"ref":"rotate_keys.html","title":"Rotate Keys","type":"extras","doc":"How to Rotate Keys Many organizations rotate their keys on a regular schedule for security reasons. Cloak can do this with zero downtime."},{"ref":"rotate_keys.html#add-your-new-key","title":"Rotate Keys - Add Your New Key","type":"extras","doc":"You will need to add your new key to your vault configuration, as the first key in the :ciphers list. Demote and re-label existing keys. For example, if your config currently looks like this: config :my_app, MyApp.Vault, ciphers: [ default: {Cloak.Ciphers.AES.GCM, tag: &quot;AES.GCM.V1&quot;, key: &lt;&lt;...&gt;&gt;}, ] Then change the :default label to the new key, and demote the existing key to the :retired label. config :my_app, MyApp.Vault, ciphers: [ default: {Cloak.Ciphers.AES.GCM, tag: &quot;AES.GCM.V2&quot;, key: &lt;&lt;...&gt;&gt;}, retired: {Cloak.Ciphers.AES.GCM, tag: &quot;AES.GCM.V1&quot;, key: &lt;&lt;...&gt;&gt;} ]"},{"ref":"rotate_keys.html#migrate-data-to-new-key","title":"Rotate Keys - Migrate Data To New Key","type":"extras","doc":"For each schema that uses your vault, run mix cloak.migrate: mix cloak.migrate.ecto -r MyApp.Repo -s MyApp.Schema Alternatively, you can migrate multiple schemas at once by configuring the following values in config/config.exs: config :my_app, cloak_repo: MyApp.Repo, cloak_schemas: [...] With that in place, you can simply run: mix cloak.migrate.ecto"},{"ref":"rotate_keys.html#remove-retired-key","title":"Rotate Keys - Remove Retired Key","type":"extras","doc":"Once you are confident that all data have been converted over to use the new key, you may remove the :retired key from your configuration. config :my_app, MyApp.Vault, ciphers: [ default: {Cloak.Ciphers.AES.GCM, tag: &quot;AES.GCM.V2&quot;, key: &lt;&lt;...&gt;&gt;}, ] How It Works Every ciphertext generated by Cloak is tagged with the :tag of the configured cipher that produced it. As a result, Cloak will be able to find and use the correct cipher to decrypt any old ciphertext as long as you leave its cipher configured. When you add a new default key, Cloak will immediately begin encrypting new data with the new key. Every time existing data are fetched from the database, they will be decrypted with the old key, and if changed, will be encrypted with the new key. Even if you don&#39;t run mix cloak.migrate.ecto, data will gradually be converted over to the new key as they are used. The migration task simply speeds up the process."},{"ref":"0-9-x_to_1-0-x.html","title":"0.9.x to 1.0.x","type":"extras","doc":"How To Upgrade From 0.9.x to 1.0.x Cloak 1.0.0 separated the Ecto components into a separate package: cloak_ecto. This required some breaking name changes. Cloak&#39;s default JSON library is now Jason, not Poison All Ecto-related modules now have the prefix Cloak.Ecto. Cloak&#39;s Ecto types have a new naming scheme: Cloak.Fields.Binary became Cloak.Ecto.Binary. mix cloak.migrate has been renamed to mix cloak.migrate.ecto. Install cloak_ecto 1.0.0 Replace your cloak dependency with cloak_ecto. {:cloak_ecto, &quot;1.0.0&quot;} Configure JSON Library If you have not specified the :json_library setting in your Vault, and want to continue using Poison, you will need to add that setting: config :my_app, MyApp.Vault, json_library: Poison, # ... Otherwise, you will need to add Jason to your dependencies: {:jason, &quot;~&gt; 1.0&quot;} Update Cloak Field References Update all your Cloak field references from the old naming scheme to the new. For example: use Cloak.Fields.Binary, vault: MyApp.Vault Should now become: use Cloak.Ecto.Binary, vault: MyApp.Vault Update Migration References If you use Cloak.Migrator directly, rename your references to Cloak.Ecto.Migrator instead. If you use mix cloak.migrate, rename any references to mix cloak.migrate.ecto instead.."}]